﻿using AircraftPlantContracts.BindingModels;
using AircraftPlantContracts.ViewModels;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AircraftPlantContracts.BusinessLogicsContracts
{
    /// <summary>
    /// Интерфейс для описания работы с отчетами
    /// </summary>
    public interface IReportLogic
    {
        /// <summary>
        /// Получение списка изделий с указанием используемых компонентов
        /// </summary>
        /// <returns></returns>
        List<ReportPlaneComponentViewModel> GetPlaneComponents();

        /// <summary>
        /// Получение списка заказов за определенный период
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        List<ReportOrdersViewModel> GetOrders(ReportBindingModel model);

        /// <summary>
        /// Получение списка заказов с группировкой по датам
        /// </summary>
        /// <returns></returns>
        List<ReportGroupOrdersViewModel> GetGroupOrders();

        /// <summary>
        /// Получение списка магазинов с указанием хранимых изделий
        /// </summary>
        /// <returns></returns>
        List<ReportShopPlanesViewModel> GetShopPlanes();

        /// <summary>
        /// Сохранение изделий в файл-Word
        /// </summary>
        /// <param name="model"></param>
        void SavePlanesToWordFile(ReportBindingModel model);

        /// <summary>
        /// Сохранение изделий с указаеним компонентов в файл-Excel
        /// </summary>
        /// <param name="model"></param>
        void SavePlaneComponentsToExcelFile(ReportBindingModel model);

        /// <summary>
        /// Сохранение заказов в файл-Pdf
        /// </summary>
        /// <param name="model"></param>
        void SaveOrdersToPdfFile(ReportBindingModel model);

        /// <summary>
        /// Сохранение магазинов в Word-файл
        /// </summary>
        /// <param name="model"></param>
        void SaveShopsToWordFile(ReportBindingModel model);

        /// <summary>
        /// Сохранение магазинов с указанием изделий в файл-Excel
        /// </summary>
        /// <param name="model"></param>
        void SaveShopPlanesToExcelFile(ReportBindingModel model);

        /// <summary>
        /// Сохранение заказов с группировкой по датам в файл-Pdf
        /// </summary>
        /// <param name="model"></param>
        void SaveGroupOrdersToPdfFile(ReportBindingModel model);
    }
}
