﻿using AircraftPlantContracts.BindingModels;
using AircraftPlantContracts.SearchModels;
using AircraftPlantContracts.ViewModels;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AircraftPlantContracts.StoragesContracts
{
    /// <summary>
    /// Интерфейс для описания работы с хранилищем для компонентов
    /// </summary>
    public interface IComponentStorage
    {
        /// <summary>
        /// Получение полного списка
        /// </summary>
        /// <returns></returns>
        List<ComponentViewModel> GetFullList();

        /// <summary>
        /// Получение фильтрованного списка
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        List<ComponentViewModel> GetFilteredList(ComponentSearchModel model);

        /// <summary>
        /// Получение элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        ComponentViewModel? GetElement(ComponentSearchModel model);

        /// <summary>
        /// Добавление элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        ComponentViewModel? Insert(ComponentBindingModel model);

        /// <summary>
        /// Редактирование элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        ComponentViewModel? Update(ComponentBindingModel model);

        /// <summary>
        /// Удаление элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        ComponentViewModel? Delete(ComponentBindingModel model);
    }
}
