﻿using AircraftPlantContracts.BindingModels;
using AircraftPlantContracts.SearchModels;
using AircraftPlantContracts.ViewModels;
using AircraftPlantDataModels.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AircraftPlantContracts.StoragesContracts
{
    /// <summary>
    /// Интерфейс для описания работы с хранилищем для магазинов
    /// </summary>
    public interface IShopStorage
    {
        /// <summary>
        /// Получение полного списка
        /// </summary>
        /// <returns></returns>
        List<ShopViewModel> GetFullList();

        /// <summary>
        /// Получение фильтрованного списка
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        List<ShopViewModel> GetFilteredList(ShopSearchModel model);

        /// <summary>
        /// Получение элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        ShopViewModel? GetElement(ShopSearchModel model);

        /// <summary>
        /// Добавление элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        ShopViewModel? Insert(ShopBindingModel model);

        /// <summary>
        /// Редактирование элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        ShopViewModel? Update(ShopBindingModel model);

        /// <summary>
        /// Удаление элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        ShopViewModel? Delete(ShopBindingModel model);

        /// <summary>
        /// Продажа изделий
        /// </summary>
        /// <param name="model"></param>
        /// <param name="count"></param>
        /// <returns></returns>
        bool SellPlanes(IPlaneModel model, int count);

        /// <summary>
        /// Проверка наличия в нужном количестве
        /// </summary>
        /// <param name="model"></param>
        /// <param name="count"></param>
        /// <returns></returns>
        bool CheckCount(IPlaneModel model, int count);
    }
}
