﻿using AircraftPlantDataModels.Enums;
using AircraftPlantDataModels.Models;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AircraftPlantContracts.ViewModels
{
    /// <summary>
    /// Модель для передачи данных пользователю
    /// для отображения для заказов
    /// </summary>
    public class OrderViewModel : IOrderModel
    {
        /// <summary>
        /// Идентификатор
        /// </summary>
        [DisplayName("Номер")]
        public int Id { get; set; }

        /// <summary>
        /// Идентификатор изделия
        /// </summary>
        public int PlaneId { get; set; }

        /// <summary>
        /// Название изделия
        /// </summary>
        [DisplayName("Изделие")]
        public string PlaneName { get; set; } = string.Empty;

        /// <summary>
        /// Идентификатор клиента
        /// </summary>
        public int ClientId { get; set; }

        /// <summary>
        /// ФИО клиента
        /// </summary>
        [DisplayName("ФИО клиента")]
        public string ClientFIO { get; set; } = string.Empty;

        /// <summary>
        /// Идентификатор исполнителя
        /// </summary>
        public int? ImplementerId { get; set; }

        /// <summary>
        /// ФИО клиента
        /// </summary>
        [DisplayName("ФИО исполнителя")]
        public string ImplementerFIO { get; set; } = string.Empty;

        /// <summary>
        /// Количество изделий
        /// </summary>
        [DisplayName("Количество")]
        public int Count { get; set; }

        /// <summary>
        /// Сумма заказа
        /// </summary>
        [DisplayName("Сумма")]
        public double Sum { get; set; }

        /// <summary>
        /// Статус заказа
        /// </summary>
        [DisplayName("Статус")]
        public OrderStatus Status { get; set; } = OrderStatus.Неизвестен;

        /// <summary>
        /// Дата создания заказа
        /// </summary>
        [DisplayName("Дата создания")]
        public DateTime DateCreate { get; set; } = DateTime.Now;

        /// <summary>
        /// Дата выполнения заказа
        /// </summary>
        [DisplayName("Дата выполнения")]
        public DateTime? DateImplement { get; set; }
    }
}
