﻿using AircraftPlantContracts.BindingModels;
using AircraftPlantContracts.SearchModels;
using AircraftPlantContracts.StoragesContracts;
using AircraftPlantContracts.ViewModels;
using AircraftPlantDatabaseImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AircraftPlantDatabaseImplement.Implements
{
    /// <summary>
    /// Реализация интерфейса хранилища для клиентов
    /// </summary>
    public class ClientStorage : IClientStorage
    {
        /// <summary>
        /// Получение полного списка
        /// </summary>
        /// <returns></returns>
        public List<ClientViewModel> GetFullList()
        {
            using var context = new AircraftPlantDatabase();
            return context.Clients
                .Select(x => x.GetViewModel)
                .ToList();
        }

        /// <summary>
        /// Получение фильтрованного списка
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public List<ClientViewModel> GetFilteredList(ClientSearchModel model)
        {
            if (string.IsNullOrEmpty(model.Email))
            {
                return new();
            }

            using var context = new AircraftPlantDatabase();
            return context.Clients
                .Where(x => x.Email.Contains(model.Email))
                .Select(x => x.GetViewModel)
                .ToList();
        }

        /// <summary>
        /// Получение элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public ClientViewModel? GetElement(ClientSearchModel model)
        {
            using var context = new AircraftPlantDatabase();
            if (model.Id.HasValue)
                return context.Clients
                    .FirstOrDefault(x => x.Id == model.Id)?
                    .GetViewModel;

            if (!string.IsNullOrEmpty(model.Email) && !string.IsNullOrEmpty(model.Password))
                return context.Clients
                    .FirstOrDefault(x => x.Email.Equals(model.Email)
                        && x.Password.Equals(model.Password))?
                    .GetViewModel;

            if (!string.IsNullOrEmpty(model.Email))
                return context.Clients
                    .FirstOrDefault(x => x.Email.Equals(model.Email))?.GetViewModel;

            return null;
        }

        /// <summary>
        /// Добавление элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public ClientViewModel? Insert(ClientBindingModel model)
        {
            var newClient = Client.Create(model);
            if (newClient == null)
            {
                return null;
            }

            using var context = new AircraftPlantDatabase();
            context.Clients.Add(newClient);
            context.SaveChanges();
            return newClient.GetViewModel;
        }

        /// <summary>
        /// Редактирование элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public ClientViewModel? Update(ClientBindingModel model)
        {
            using var context = new AircraftPlantDatabase();
            var client = context.Clients.FirstOrDefault(x => x.Id == model.Id);
            if (client == null)
            {
                return null;
            }

            client.Update(model);
            context.SaveChanges();
            return client.GetViewModel;
        }

        /// <summary>
        /// Удаление элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public ClientViewModel? Delete(ClientBindingModel model)
        {
            using var context = new AircraftPlantDatabase();
            var element = context.Clients.FirstOrDefault(rec => rec.Id == model.Id);
            if (element == null)
            {
                return null;
            }

            context.Clients.Remove(element);
            context.SaveChanges();
            return element.GetViewModel;
        }
    }
}
