﻿using AircraftPlantContracts.BindingModels;
using AircraftPlantContracts.SearchModels;
using AircraftPlantContracts.StoragesContracts;
using AircraftPlantContracts.ViewModels;
using AircraftPlantFileImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AircraftPlantFileImplement.Implements
{
    /// <summary>
    /// Реализация интерфейса хранилища для исполнителей
    /// </summary>
    public class ImplementerStorage : IImplementerStorage
    {
        /// <summary>
        /// Хранилище
        /// </summary>
        private readonly DataFileSingleton _source;

        /// <summary>
        /// Конструктор
        /// </summary>
        public ImplementerStorage()
        {
            _source = DataFileSingleton.GetInstance();
        }

        /// <summary>
        /// Получение полного списка
        /// </summary>
        /// <returns></returns>
        public List<ImplementerViewModel> GetFullList()
        {
            return _source.Implementers
                .Select(x => x.GetViewModel)
                .ToList();
        }

        /// <summary>
        /// Получение фильтрованного списка
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public List<ImplementerViewModel> GetFilteredList(ImplementerSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ImplementerFIO))
            {
                return new();
            }

            return _source.Implementers
                .Where(x => x.ImplementerFIO.Contains(model.ImplementerFIO))
                .Select(x => x.GetViewModel)
                .ToList();
        }

        /// <summary>
        /// Получение элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public ImplementerViewModel? GetElement(ImplementerSearchModel model)
        {
            if (!string.IsNullOrEmpty(model.ImplementerFIO) || model.Id.HasValue)
            {
                return _source.Implementers.FirstOrDefault(x =>
                    (!string.IsNullOrEmpty(model.ImplementerFIO) && x.ImplementerFIO == model.ImplementerFIO) ||
                    (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
            }

            else if (!string.IsNullOrEmpty(model.ImplementerFIO) && !string.IsNullOrEmpty(model.Password))
                return _source.Implementers.FirstOrDefault(x =>
                    x.ImplementerFIO == model.ImplementerFIO && x.Password == model.Password)?.GetViewModel;
            
            return null;
        }

        /// <summary>
        /// Добавление элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public ImplementerViewModel? Insert(ImplementerBindingModel model)
        {
            model.Id = _source.Implementers.Count > 0 ? _source.Implementers.Max(x => x.Id) + 1 : 1;
            
            var newImplementer = Implementer.Create(model);
            if (newImplementer == null)
            {
                return null;
            }

            _source.Implementers.Add(newImplementer);
            _source.SaveImplementers();
            return newImplementer.GetViewModel;
        }

        /// <summary>
        /// Редактирование элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public ImplementerViewModel? Update(ImplementerBindingModel model)
        {
            var implementer = _source.Implementers.FirstOrDefault(x => x.Id == model.Id);
            if (implementer == null)
            {
                return null;
            }

            implementer.Update(model);
            _source.SaveImplementers();
            return implementer.GetViewModel;
        }

        /// <summary>
        /// Удаление элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public ImplementerViewModel? Delete(ImplementerBindingModel model)
        {
            var element = _source.Implementers.FirstOrDefault(x => x.Id == model.Id);
            if (element != null)
            {
                _source.Implementers.Remove(element);
                _source.SaveImplementers();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
