﻿using AircraftPlantContracts.BindingModels;
using AircraftPlantContracts.ViewModels;
using AircraftPlantDataModels.Enums;
using AircraftPlantDataModels.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace AircraftPlantFileImplement.Models
{
    /// <summary>
    /// Сущность "Заказ"
    /// </summary>
    public class Order : IOrderModel
    {
        /// <summary>
        /// Идентификатор
        /// </summary>
        public int Id { get; private set; }

        /// <summary>
        /// Идентификатор изделия
        /// </summary>
        public int PlaneId { get; private set; }

        /// <summary>
        /// Идентификатор клиента
        /// </summary>
        public int ClientId { get; private set; }

        /// <summary>
        /// Идентификатор исполнителя
        /// </summary>
        public int? ImplementerId { get; private set; }

        /// <summary>
        /// Количество изделий
        /// </summary>
        public int Count { get; private set; }

        /// <summary>
        /// Сумма заказа
        /// </summary>
        public double Sum { get; private set; }

        /// <summary>
        /// Статус заказа
        /// </summary>
        public OrderStatus Status { get; private set; }

        /// <summary>
        /// Дата создания заказа
        /// </summary>
        public DateTime DateCreate { get; private set; }

        /// <summary>
        /// Дата выполнения заказа
        /// </summary>
        public DateTime? DateImplement { get; private set; }

        /// <summary>
        /// Создание модели заказа
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public static Order? Create(OrderBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }

            return new Order
            {
                Id = model.Id,
                PlaneId = model.PlaneId,
                ClientId = model.ClientId,
                ImplementerId = model.ImplementerId,
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement
            };
        }

        /// <summary>
        /// Создание модели заказа из данных файла
        /// </summary>
        /// <param name="element"></param>
        /// <returns></returns>
        public static Order? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }

            return new Order()
            {
                Id = Convert.ToInt32(element.Attribute("Id")!.Value),
                PlaneId = Convert.ToInt32(element.Element("PlaneId")!.Value),
                ClientId = Convert.ToInt32(element.Element("ClientId")!.Value),
                ImplementerId = Convert.ToInt32(element.Element("ImplementerId")!.Value),
                Sum = Convert.ToDouble(element.Element("Sum")!.Value),
                Count = Convert.ToInt32(element.Element("Count")!.Value),
                Status = (OrderStatus)Enum.Parse(typeof(OrderStatus), element.Element("Status")!.Value),
                DateCreate = Convert.ToDateTime(element.Element("DateCreate")!.Value),
                DateImplement = string.IsNullOrEmpty(element.Element("DateImplement")!.Value) ? null : Convert.ToDateTime(element.Element("DateImplement")!.Value)
            };
        }

        /// <summary>
        /// Изменение модели заказа
        /// </summary>
        /// <param name="model"></param>
        public void Update(OrderBindingModel? model)
        {
            if (model == null)
            {
                return;
            }

            Status = model.Status;
            DateImplement = model.DateImplement;
            if (model.ImplementerId.HasValue)
                ImplementerId = model.ImplementerId;
        }

        /// <summary>
        /// Получение модели заказа
        /// </summary>
        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            PlaneId = PlaneId,
            ClientId = ClientId,
            ImplementerId = ImplementerId,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement
        };

        /// <summary>
        /// Запись данных о модели заказа в файл
        /// </summary>
        public XElement GetXElement => new("Order",
            new XAttribute("Id", Id),
            new XElement("PlaneId", PlaneId),
            new XElement("ClientId", ClientId),
            new XElement("ImplementerId", ImplementerId),
            new XElement("Count", Count.ToString()),
            new XElement("Sum", Sum.ToString()),
            new XElement("Status", Status.ToString()),
            new XElement("DateCreate", DateCreate.ToString()),
            new XElement("DateImplement", DateImplement.ToString()));
    }
}
