﻿using AircraftPlantListImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AircraftPlantListImplement
{
    /// <summary>
    /// Класс для хранения списков классов-моделей (паттерн Singleton)
    /// </summary>
    public class DataListSingleton
    {
        /// <summary>
        /// Ссылка на класс
        /// </summary>
        private static DataListSingleton? _instance;

        /// <summary>
        /// Список классов-моделей компонентов
        /// </summary>
        public List<Component> Components { get; set; }
        
        /// <summary>
        /// Список классов-моделей заказов
        /// </summary>
        public List<Order> Orders { get; set; }

        /// <summary>
        /// Список классов-моделей изделий
        /// </summary>
        public List<Plane> Planes { get; set; }

        /// <summary>
        /// Список классов-моделей магазинов
        /// </summary>
        public List<Shop> Shops { get; set; }

        /// <summary>
        /// Список классов-моделей клиентов
        /// </summary>
        public List<Client> Clients { get; set; }

        /// <summary>
        /// Список классов-моделей исполнителей
        /// </summary>
        public List<Implementer> Implementers { get; set; }

        /// <summary>
        /// Конструктор
        /// </summary>
        private DataListSingleton()
        {
            Components = new List<Component>();
            Orders = new List<Order>();
            Planes = new List<Plane>();
            Shops = new List<Shop>();
            Clients = new List<Client>();
            Implementers = new List<Implementer>();
        }

        /// <summary>
        /// Получить ссылку на класс
        /// </summary>
        /// <returns></returns>
        public static DataListSingleton GetInstance()
        {
            if (_instance == null)
            {
                _instance = new DataListSingleton();
            }
            return _instance;
        }
    }
}
