﻿using AircraftPlantContracts.BindingModels;
using AircraftPlantContracts.SearchModels;
using AircraftPlantContracts.StoragesContracts;
using AircraftPlantContracts.ViewModels;
using AircraftPlantListImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AircraftPlantListImplement.Implements
{
    /// <summary>
    /// Реализация интерфейса хранилища для клиентов
    /// </summary>
    public class ClientStorage : IClientStorage
    {
        /// <summary>
        /// Хранилище
        /// </summary>
        private readonly DataListSingleton _source;

        /// <summary>
        /// Конструктор
        /// </summary>
        public ClientStorage()
        {
            _source = DataListSingleton.GetInstance();
        }

        /// <summary>
        /// Получение полного списка
        /// </summary>
        /// <returns></returns>
        public List<ClientViewModel> GetFullList()
        {
            var result = new List<ClientViewModel>();
            foreach (var client in _source.Clients)
            {
                result.Add(client.GetViewModel);
            }
            return result;
        }

        /// <summary>
        /// Получение фильтрованного списка
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public List<ClientViewModel> GetFilteredList(ClientSearchModel model)
        {
            var result = new List<ClientViewModel>();

            if (string.IsNullOrEmpty(model.Email))
            {
                return result;
            }

            foreach (var client in _source.Clients)
            {
                if (client.Email.Contains(model.Email))
                {
                    result.Add(client.GetViewModel);
                }
            }
            return result;
        }

        /// <summary>
        /// Получение элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public ClientViewModel? GetElement(ClientSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ClientFIO) && string.IsNullOrEmpty(model.Email) && string.IsNullOrEmpty(model.Password) && !model.Id.HasValue)
            {
                return null;
            }

            foreach (var client in _source.Clients)
            {
                if (model.Id.HasValue && model.Id == client.Id)
                    return client.GetViewModel;
                if (!string.IsNullOrEmpty(model.Email) && !string.IsNullOrEmpty(model.Password) &&
                    client.Email.Equals(model.Email) && client.Password.Equals(model.Password))
                    return client.GetViewModel;
                if (!string.IsNullOrEmpty(model.Email) && client.Email.Equals(model.Email))
                    return client.GetViewModel;
            }
            return null;
        }

        /// <summary>
        /// Добавление элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public ClientViewModel? Insert(ClientBindingModel model)
        {
            model.Id = 1;
            foreach (var client in _source.Clients)
            {
                if (model.Id <= client.Id)
                {
                    model.Id = client.Id + 1;
                }
            }

            var newClient = Client.Create(model);
            if (newClient == null)
            {
                return null;
            }

            _source.Clients.Add(newClient);
            return newClient.GetViewModel;
        }

        /// <summary>
        /// Редактирование элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public ClientViewModel? Update(ClientBindingModel model)
        {
            foreach (var client in _source.Clients)
            {
                if (client.Id == model.Id)
                {
                    client.Update(model);
                    return client.GetViewModel;
                }
            }
            return null;
        }

        /// <summary>
        /// Удаление элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public ClientViewModel? Delete(ClientBindingModel model)
        {
            for (int i = 0; i < _source.Clients.Count; ++i)
            {
                if (_source.Clients[i].Id == model.Id)
                {
                    var element = _source.Clients[i];
                    _source.Clients.RemoveAt(i);
                    return element.GetViewModel;
                }
            }
            return null;
        }
    }
}
