﻿using AircraftPlantContracts.BindingModels;
using AircraftPlantContracts.SearchModels;
using AircraftPlantContracts.StoragesContracts;
using AircraftPlantContracts.ViewModels;
using AircraftPlantListImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AircraftPlantListImplement.Implements
{
    /// <summary>
    /// Реализация интерфейса хранилища для исполнителей
    /// </summary>
    public class ImplementerStorage : IImplementerStorage
    {
        /// <summary>
        /// Хранилище
        /// </summary>
        private readonly DataListSingleton _source;

        /// <summary>
        /// Конструктор
        /// </summary>
        public ImplementerStorage()
        {
            _source = DataListSingleton.GetInstance();
        }

        /// <summary>
        /// Получение полного списка
        /// </summary>
        /// <returns></returns>
        public List<ImplementerViewModel> GetFullList()
        {
            var result = new List<ImplementerViewModel>();
            foreach (var implementer in _source.Implementers)
            {
                result.Add(implementer.GetViewModel);
            }
            return result;
        }

        /// <summary>
        /// Получение фильтрованного списка
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public List<ImplementerViewModel> GetFilteredList(ImplementerSearchModel model)
        {
            var result = new List<ImplementerViewModel>();
            if (string.IsNullOrEmpty(model.ImplementerFIO))
            {
                return result;
            }

            foreach (var implementer in _source.Implementers)
            {
                if (implementer.ImplementerFIO.Contains(model.ImplementerFIO))
                {
                    result.Add(implementer.GetViewModel);
                }
            }
            return result;
        }

        /// <summary>
        /// Получение элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public ImplementerViewModel? GetElement(ImplementerSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ImplementerFIO) && string.IsNullOrEmpty(model.Password) && !model.Id.HasValue)
            {
                return null;
            }

            foreach (var implementer in _source.Implementers)
            {
                if (model.Id.HasValue && model.Id == implementer.Id)
                    return implementer.GetViewModel;
                if (!string.IsNullOrEmpty(model.ImplementerFIO) && !string.IsNullOrEmpty(model.Password) &&
                    implementer.ImplementerFIO.Equals(model.ImplementerFIO) && implementer.Password.Equals(model.Password))
                    return implementer.GetViewModel;
                if (!string.IsNullOrEmpty(model.ImplementerFIO) && implementer.ImplementerFIO.Equals(model.ImplementerFIO))
                    return implementer.GetViewModel;
            }
            return null;
        }

        /// <summary>
        /// Добавление элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public ImplementerViewModel? Insert(ImplementerBindingModel model)
        {
            model.Id = 1;
            foreach (var implementer in _source.Implementers)
            {
                if (model.Id <= implementer.Id)
                {
                    model.Id = implementer.Id + 1;
                }
            }

            var newImplementer = Implementer.Create(model);
            if (newImplementer == null)
            {
                return null;
            }

            _source.Implementers.Add(newImplementer);
            return newImplementer.GetViewModel;
        }

        /// <summary>
        /// Редактирование элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public ImplementerViewModel? Update(ImplementerBindingModel model)
        {
            foreach (var implementer in _source.Implementers)
            {
                if (implementer.Id == model.Id)
                {
                    implementer.Update(model);
                    return implementer.GetViewModel;
                }
            }
            return null;
        }

        /// <summary>
        /// Удаление элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public ImplementerViewModel? Delete(ImplementerBindingModel model)
        {
            for (int i = 0; i < _source.Implementers.Count; ++i)
            {
                if (_source.Implementers[i].Id == model.Id)
                {
                    var element = _source.Implementers[i];
                    _source.Implementers.RemoveAt(i);
                    return element.GetViewModel;
                }
            }
            return null;
        }
    }
}
