﻿using AircraftPlantContracts.BindingModels;
using AircraftPlantContracts.SearchModels;
using AircraftPlantContracts.StoragesContracts;
using AircraftPlantContracts.ViewModels;
using AircraftPlantListImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AircraftPlantListImplement.Implements
{
    /// <summary>
    /// Реализация интерфейса хранилища для изделий
    /// </summary>
    public class PlaneStorage : IPlaneStorage
    {
        /// <summary>
        /// Хранилище
        /// </summary>
        private readonly DataListSingleton _source;

        /// <summary>
        /// Конструктор
        /// </summary>
        public PlaneStorage()
        {
            _source = DataListSingleton.GetInstance();
        }

        /// <summary>
        /// Получение полного списка
        /// </summary>
        /// <returns></returns>
        public List<PlaneViewModel> GetFullList()
        {
            var result = new List<PlaneViewModel>();
            foreach (var plane in _source.Planes)
            {
                result.Add(plane.GetViewModel);
            }
            return result;
        }

        /// <summary>
        /// Получение фильтрованного списка
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public List<PlaneViewModel> GetFilteredList(PlaneSearchModel model)
        {
            var result = new List<PlaneViewModel>();
            if (string.IsNullOrEmpty(model.PlaneName))
            {
                return result;
            }

            foreach (var plane in _source.Planes)
            {
                if (plane.PlaneName.Contains(model.PlaneName))
                {
                    result.Add(plane.GetViewModel);
                }
            }
            return result;
        }

        /// <summary>
        /// Получение элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public PlaneViewModel? GetElement(PlaneSearchModel model)
        {
            if (string.IsNullOrEmpty(model.PlaneName) && !model.Id.HasValue)
            {
                return null;
            }

            foreach (var plane in _source.Planes)
            {
                if ((!string.IsNullOrEmpty(model.PlaneName) && plane.PlaneName == model.PlaneName) || (model.Id.HasValue && plane.Id == model.Id))
                {
                    return plane.GetViewModel;
                }
            }
            return null;
        }

        /// <summary>
        /// Добавление элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public PlaneViewModel? Insert(PlaneBindingModel model)
        {
            model.Id = 1;
            foreach (var plane in _source.Planes)
            {
                if (model.Id <= plane.Id)
                {
                    model.Id = plane.Id + 1;
                }
            }

            var newPlane = Plane.Create(model);
            if (newPlane == null)
            {
                return null;
            }

            _source.Planes.Add(newPlane);
            return newPlane.GetViewModel;
        }

        /// <summary>
        /// Редактирование элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public PlaneViewModel? Update(PlaneBindingModel model)
        {
            foreach (var plane in _source.Planes)
            {
                if (plane.Id == model.Id)
                {
                    plane.Update(model);
                    return plane.GetViewModel;
                }
            }
            return null;
        }

        /// <summary>
        /// Удаление элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public PlaneViewModel? Delete(PlaneBindingModel model)
        {
            for (int i = 0; i < _source.Planes.Count; ++i)
            {
                if (_source.Planes[i].Id == model.Id)
                {
                    var element = _source.Planes[i];
                    _source.Planes.RemoveAt(i);
                    return element.GetViewModel;
                }
            }
            return null;
        }
    }
}
