﻿using AircraftPlantContracts.BindingModels;
using AircraftPlantContracts.ViewModels;
using AircraftPlantDataModels.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AircraftPlantListImplement.Models
{
    /// <summary>
    /// Сущность "Исполнитель"
    /// </summary>
    public class Implementer : IImplementerModel
    {
        /// <summary>
        /// Идентификатор
        /// </summary>
        public int Id { get; private set; }

        /// <summary>
        /// ФИО исполнителя
        /// </summary>
        public string ImplementerFIO { get; private set; } = string.Empty;

        /// <summary>
        /// Пароль
        /// </summary>
        public string Password { get; private set; } = string.Empty;

        /// <summary>
        /// Опыт работы
        /// </summary>
        public int WorkExperience { get; private set; }

        /// <summary>
        /// Квалификация
        /// </summary>
        public int Qualification { get; private set; }

        /// <summary>
        /// Создание модели исполнителя
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public static Implementer? Create(ImplementerBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }

            return new Implementer()
            {
                Id = model.Id,
                ImplementerFIO = model.ImplementerFIO,
                Password = model.Password,
                WorkExperience = model.WorkExperience,
                Qualification = model.Qualification,
            };
        }

        /// <summary>
        /// Изменение модели исполнителя
        /// </summary>
        /// <param name="model"></param>
        public void Update(ImplementerBindingModel? model)
        {
            if (model == null)
            {
                return;
            }

            ImplementerFIO = model.ImplementerFIO;
            Password = model.Password;
            WorkExperience = model.WorkExperience;
            Qualification = model.Qualification;
        }

        /// <summary>
        /// Получение модели исполнителя
        /// </summary>
        public ImplementerViewModel GetViewModel => new()
        {
            Id = Id,
            ImplementerFIO = ImplementerFIO,
            Password = Password,
            WorkExperience = WorkExperience,
            Qualification = Qualification
        };
    }
}
