﻿using AircraftPlantContracts.BindingModels;
using AircraftPlantContracts.ViewModels;
using AircraftPlantDataModels.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AircraftPlantListImplement.Models
{
    /// <summary>
    /// Сущность "Магазин"
    /// </summary>
    public class Shop : IShopModel
    {
        /// <summary>
        /// Идентификатор
        /// </summary>
        public int Id { get; private set; }

        /// <summary>
        /// Название магазина
        /// </summary>
        public string ShopName { get; private set; } = string.Empty;

        /// <summary>
        /// Адрес магазина
        /// </summary>
        public string Address { get; private set; } = string.Empty;

        /// <summary>
        /// Дата открытия магазина
        /// </summary>
        public DateTime DateOpening { get; private set; }

        /// <summary>
        /// Коллекция изделий в магазине
        /// </summary>
        public Dictionary<int, (IPlaneModel, int)> ShopPlanes
        {
            get;
            private set;
        } = new Dictionary<int, (IPlaneModel, int)>();

        /// <summary>
        /// Максимальное количество изделий
        /// </summary>
        public int MaxPlanes { get; private set; }

        /// <summary>
        /// Создание модели магазина
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public static Shop? Create(ShopBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }

            return new Shop()
            {
                Id = model.Id,
                ShopName = model.ShopName,
                Address = model.Address,
                DateOpening = model.DateOpening,
                ShopPlanes = model.ShopPlanes,
                MaxPlanes = model.MaxPlanes
            };
        }

        /// <summary>
        /// Изменение модели магазина
        /// </summary>
        /// <param name="model"></param>
        public void Update(ShopBindingModel? model)
        {
            if (model == null)
            {
                return;
            }

            ShopName = model.ShopName;
            Address = model.Address;
            DateOpening = model.DateOpening;
            ShopPlanes = model.ShopPlanes;
            MaxPlanes = model.MaxPlanes;
        }
        
        /// <summary>
        /// Получение модели магазина
        /// </summary>
        public ShopViewModel GetViewModel => new()
        {
            Id = Id,
            ShopName = ShopName,
            Address = Address,
            DateOpening = DateOpening,
            ShopPlanes = ShopPlanes,
            MaxPlanes = MaxPlanes
        };
    }
}
