﻿using AircraftPlantContracts.BindingModels;
using AircraftPlantContracts.BusinessLogicsContracts;
using AircraftPlantContracts.SearchModels;
using AircraftPlantContracts.ViewModels;
using Microsoft.AspNetCore.Mvc;

namespace AircraftPlantRestApi.Controllers
{
    /// <summary>
    /// Контроллер для работы с клиентами
    /// </summary>
    [Route("api/[controller]/[action]")]
    [ApiController]
    public class ClientController : Controller
    {
        /// <summary>
        /// Логгер
        /// </summary>
        private readonly ILogger _logger;

        /// <summary>
        /// Бизнес-логика для клиентов
        /// </summary>
        private readonly IClientLogic _logic;

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="logic"></param>
        /// <param name="logger"></param>
        public ClientController(IClientLogic logic, ILogger<ClientController> logger)
        {
            _logger = logger;
            _logic = logic;
        }

        /// <summary>
        /// Вход в систему
        /// </summary>
        /// <param name="login"></param>
        /// <param name="password"></param>
        /// <returns></returns>
        [HttpGet]
        public ClientViewModel? Login(string login, string password)
        {
            try
            {
                return _logic.ReadElement(new ClientSearchModel
                {
                    Email = login,
                    Password = password
                });
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка входа в систему");
                throw;
            }
        }

        /// <summary>
        /// Регистрация
        /// </summary>
        /// <param name="model"></param>
        [HttpPost]
        public void Register(ClientBindingModel model)
        {
            try
            {
                _logic.Create(model);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка регистрации");
                throw;
            }
        }

        /// <summary>
        /// Обновление данных
        /// </summary>
        /// <param name="model"></param>
        [HttpPost]
        public void UpdateData(ClientBindingModel model)
        {
            try
            {
                _logic.Update(model);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка обновления данных");
                throw;
            }
        }
    }
}
