﻿using Newtonsoft.Json;
using System.Net.Http.Headers;
using System.Text;

namespace AircraftPlantShopApp
{
    /// <summary>
    /// API-клиент
    /// </summary>
    public class APIClient
    {
        /// <summary>
        /// Http-клиент
        /// </summary>
        private static readonly HttpClient _client = new();

        /// <summary>
        /// Пароль
        /// </summary>
        public static string Password { get; private set; } = string.Empty;

        /// <summary>
        /// Доступ
        /// </summary>
        public static bool Access { get; private set; } = false;

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="configuration"></param>
        public static void Connect(IConfiguration configuration)
        {
            Password = configuration["Password"];
            _client.BaseAddress = new Uri(configuration["IPAddress"]);
            _client.DefaultRequestHeaders.Accept.Clear();
            _client.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));
        }

        /// <summary>
        /// Get-запрос
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="requestUrl"></param>
        /// <returns></returns>
        /// <exception cref="Exception"></exception>
        public static T? GetRequest<T>(string requestUrl)
        {
            var response = _client.GetAsync(requestUrl);
            var result = response.Result.Content.ReadAsStringAsync().Result;
            if (response.Result.IsSuccessStatusCode)
            {
                return JsonConvert.DeserializeObject<T>(result);
            }
            else
            {
                throw new Exception(result);
            }
        }

        /// <summary>
        /// Post-запрос
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="requestUrl"></param>
        /// <param name="model"></param>
        /// <exception cref="Exception"></exception>
        public static void PostRequest<T>(string requestUrl, T model)
        {
            var json = JsonConvert.SerializeObject(model);
            var data = new StringContent(json, Encoding.UTF8, "application/json");

            var response = _client.PostAsync(requestUrl, data);

            var result = response.Result.Content.ReadAsStringAsync().Result;
            if (!response.Result.IsSuccessStatusCode)
            {
                throw new Exception(result);
            }
        }

        /// <summary>
        /// Проверка доступа
        /// </summary>
        /// <param name="password"></param>
        /// <returns></returns>
        public static bool CheckPassword(string password)
        {
            return APIClient.Access = password == Password;
        }
    }
}
