﻿using AircraftPlantBusinessLogic.BusinessLogics;
using AircraftPlantContracts.BindingModels;
using AircraftPlantContracts.BusinessLogicsContracts;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace AircraftPlantView
{
    /// <summary>
    /// Главная форма
    /// </summary>
    public partial class FormMain : Form
    {
        /// <summary>
        /// Логгер
        /// </summary>
        private readonly ILogger _logger;

        /// <summary>
        /// Бизнес-логика для заказов
        /// </summary>
        private readonly IOrderLogic _orderLogic;

        /// <summary>
        /// Взаимодействие с отчетами
        /// </summary>
        private readonly IReportLogic _reportLogic;

        /// <summary>
        /// Имитация деятельности исполнителей
        /// </summary>
        private readonly IWorkProcess _workProcess;

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="logger"></param>
        /// <param name="logic"></param>
        public FormMain(ILogger<FormMain> logger, IOrderLogic orderLogic, IReportLogic reportLogic, IWorkProcess workProcess)
        {
            InitializeComponent();
            _logger = logger;
            _orderLogic = orderLogic;
            _reportLogic = reportLogic;
            _workProcess = workProcess;
        }

        /// <summary>
        /// Загрузка списка заказов
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void FormMain_Load(object sender, EventArgs e)
        {
            LoadData();
        }

        /// <summary>
        /// Показать список всех компонентов
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void компонентыToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormComponents));
            if (service is FormComponents form)
            {
                form.ShowDialog();
            }
        }

        /// <summary>
        /// Показать список всех изделий
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void изделияToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormPlanes));
            if (service is FormPlanes form)
            {
                form.ShowDialog();
            }
        }

        /// <summary>
        /// Показать список всех магазинов
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void магазиныToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormShops));
            if (service is FormShops form)
            {
                form.ShowDialog();
            }
        }

        /// <summary>
        /// Вывести список всех клиентов
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void клиентыToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormClients));
            if (service is FormClients form)
            {
                form.ShowDialog();
            }
        }

        /// <summary>
        /// Вывести список всех исполнителей
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void исполнителиToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormImplementers));
            if (service is FormImplementers form)
            {
                form.ShowDialog();
            }
        }

        /// <summary>
        /// Вывести отчет по изделиям
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void изделияToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            using var dialog = new SaveFileDialog { Filter = "docx|*.docx" };
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                _reportLogic.SavePlanesToWordFile(new ReportBindingModel { FileName = dialog.FileName });
                MessageBox.Show("Выполнено", "Успех", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        /// <summary>
        /// Получить отчет по изделиям с расшифровкой по компонентам
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void изделияСКомпонентамиToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormReportPlaneComponents));
            if (service is FormReportPlaneComponents form)
            {
                form.ShowDialog();
            }
        }

        /// <summary>
        /// Получить отчет по заказам
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void списокЗаказовToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormReportOrders));
            if (service is FormReportOrders form)
            {
                form.ShowDialog();
            }
        }

        /// <summary>
        /// Получить отчет по заказам с группировкой по датам
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void заказыСГруппировкойToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormReportGroupOrders));
            if (service is FormReportGroupOrders form)
            {
                form.ShowDialog();
            }
        }

        /// <summary>
        /// Получить отчет со списком магазинов
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void списокМагазиновToolStripMenuItem_Click(object sender, EventArgs e)
        {
            using var dialog = new SaveFileDialog { Filter = "docx|*.docx" };
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                _reportLogic.SaveShopsToWordFile(new ReportBindingModel { FileName = dialog.FileName });
                MessageBox.Show("Выполнено", "Успех", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        /// <summary>
        /// Получить отчет по ассортименту магазинов
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ассортиментМагазиновToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormReportShopPlanes));
            if (service is FormReportShopPlanes form)
            {
                form.ShowDialog();
            }
        }

        /// <summary>
        /// Кнопка "Создать заказ"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void запускРаботToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _workProcess.DoWork((
                Program.ServiceProvider?.GetService(typeof(IImplementerLogic)) as IImplementerLogic)!,
                _orderLogic);
            MessageBox.Show("Процесс обработки запущен", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        /// <summary>
        /// Кнопка "Создать заказ"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonCreateOrder_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormCreateOrder));
            if (service is FormCreateOrder form)
            {
                form.ShowDialog();
                LoadData();
            }
        }

        /// <summary>
        /// Кнопка "Заказ выдан"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonIssuedOrder_Click(object sender, EventArgs e)
        {
            if (dataGridView.SelectedRows.Count == 1)
            {
                int id = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["Id"].Value);
                _logger.LogInformation("Заказ №{id}. Меняется статус на 'Выдан'", id);
                try
                {
                    var operationResult = _orderLogic.FinishOrder(new OrderBindingModel { Id = id });
                    if (!operationResult)
                    {
                        throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
                    }
                    _logger.LogInformation("Заказ №{id} выдан", id);
                    LoadData();
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка отметки о выдаче заказа");
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        /// <summary>
        /// Кнопка "Добавить изделие в магазин"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonAddPlaneInShop_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormCreateSupply));
            if (service is FormCreateSupply form)
            {
                form.ShowDialog();
            }
        }

        /// <summary>
        /// Кнопка "Продать изделия"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonSellPlanes_Click(object sender, EventArgs e)
        {
            var services = Program.ServiceProvider?.GetService(typeof(FormSell));
            if (services is FormSell form)
            {
                form.ShowDialog();
            }
        }

        /// <summary>
        /// Кнопка "Обновить список"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonRefresh_Click(object sender, EventArgs e)
        {
            LoadData();
        }

        /// <summary>
        /// Метод загрузки списка заказов
        /// </summary>
        private void LoadData()
        {
            _logger.LogInformation("Загрузка заказов");
            try
            {
                var list = _orderLogic.ReadList(null);
                if (list != null)
                {
                    dataGridView.DataSource = list;
                    dataGridView.Columns["PlaneId"].Visible = false;
                    dataGridView.Columns["ClientId"].Visible = false;
                    dataGridView.Columns["ImplementerId"].Visible = false;
                }
                _logger.LogInformation("Загрузка заказов");
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки заказов");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
    }
}
