﻿using AircraftPlantContracts.BindingModels;
using AircraftPlantContracts.BusinessLogicsContracts;
using Microsoft.Extensions.Logging;
using Microsoft.Reporting.WinForms;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace AircraftPlantView
{
    /// <summary>
    /// Форма формирования отчета по заказам с группировкой по датам
    /// </summary>
    public partial class FormReportGroupOrders : Form
    {
        /// <summary>
        /// Отчет
        /// </summary>
        private readonly ReportViewer reportViewer;

        /// <summary>
        /// Логгер
        /// </summary>
        private readonly ILogger _logger;

        /// <summary>
        /// Бизнес-логика для отчетов
        /// </summary>
        private readonly IReportLogic _logic;

        /// <summary>
        /// Конструктор
        /// </summary>
        public FormReportGroupOrders(ILogger<FormReportGroupOrders> logger, IReportLogic logic)
        {
            InitializeComponent();

            _logger = logger;
            _logic = logic;

            reportViewer = new ReportViewer
            {
                Dock = DockStyle.Fill
            };
            reportViewer.LocalReport.LoadReportDefinition(new FileStream("ReportGroupOrders.rdlc", FileMode.Open));
            Controls.Clear();
            Controls.Add(reportViewer);
            Controls.Add(panelControl);
        }

        /// <summary>
        /// Кнопка "Сформировать"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonMake_Click(object sender, EventArgs e)
        {
            try
            {
                var dataSource = _logic.GetGroupOrders();
                var source = new ReportDataSource("DataSetGroupOrders", dataSource);
                reportViewer.LocalReport.DataSources.Clear();
                reportViewer.LocalReport.DataSources.Add(source);
                reportViewer.RefreshReport();
                _logger.LogInformation("Загрузка списка заказов с группировкой по датам");
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки списка заказов c группировкой по датам");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Кнопка "В Pdf"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonToPdf_Click(object sender, EventArgs e)
        {
            using var dialog = new SaveFileDialog { Filter = "pdf|*.pdf" };
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _logic.SaveGroupOrdersToPdfFile(new ReportBindingModel
                    {
                        FileName = dialog.FileName
                    });
                    _logger.LogInformation("Сохранение списка заказов с группировкой по датам");
                    MessageBox.Show("Выполнено", "Успех", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка сохранения списка заказов с группировкой по датам");
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }
    }
}
