using AircraftPlantBusinessLogic.BusinessLogics;
using AircraftPlantBusinessLogic.OfficePackage.Implements;
using AircraftPlantBusinessLogic.OfficePackage;
using AircraftPlantContracts.BusinessLogicsContracts;
using AircraftPlantContracts.StoragesContracts;
using AircraftPlantDatabaseImplement.Implements;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using NLog.Extensions.Logging;
using System;

namespace AircraftPlantView
{
    internal static class Program
    {
        /// <summary>
        /// IoC-
        /// </summary>
        private static ServiceProvider? _serviceProvider;
        public static ServiceProvider? ServiceProvider => _serviceProvider;

        /// <summary>
        ///  The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            // To customize application configuration such as set high DPI settings or default font,
            // see https://aka.ms/applicationconfiguration.
            ApplicationConfiguration.Initialize();

            var services = new ServiceCollection();
            ConfigureServices(services);
            _serviceProvider = services.BuildServiceProvider();

            Application.Run(_serviceProvider.GetRequiredService<FormMain>());
        }

        /// <summary>
        ///  IoC-  
        /// </summary>
        /// <param name="services"></param>
        private static void ConfigureServices(ServiceCollection services)
        {
            services.AddLogging(option =>
            {
                option.SetMinimumLevel(LogLevel.Information);
                option.AddNLog("nlog.config");
            });

            services.AddTransient<IClientStorage, ClientStorage>();
            services.AddTransient<IImplementerStorage, ImplementerStorage>();
            services.AddTransient<IComponentStorage, ComponentStorage>();
            services.AddTransient<IOrderStorage, OrderStorage>();
            services.AddTransient<IPlaneStorage, PlaneStorage>();
            services.AddTransient<IShopStorage, ShopStorage>();

            services.AddTransient<IClientLogic, ClientLogic>();
            services.AddTransient<IImplementerLogic, ImplementerLogic>();
            services.AddTransient<IComponentLogic, ComponentLogic>();
            services.AddTransient<IOrderLogic, OrderLogic>();
            services.AddTransient<IPlaneLogic, PlaneLogic>();
            services.AddTransient<IReportLogic, ReportLogic>();
            services.AddTransient<IShopLogic, ShopLogic>();
            services.AddTransient<IWorkProcess, WorkModeling>();

            services.AddTransient<AbstractSaveToWord, SaveToWord>();
            services.AddTransient<AbstractSaveToExcel, SaveToExcel>();
            services.AddTransient<AbstractSaveToPdf, SaveToPdf>();

            services.AddTransient<FormMain>();
            services.AddTransient<FormClients>();
            services.AddTransient<FormImplementers>();
            services.AddTransient<FormImplementer>();
            services.AddTransient<FormComponent>();
            services.AddTransient<FormComponents>();
            services.AddTransient<FormCreateOrder>();
            services.AddTransient<FormPlane>();
            services.AddTransient<FormPlaneComponent>();
            services.AddTransient<FormPlanes>();
            services.AddTransient<FormShop>();
            services.AddTransient<FormShops>();
            services.AddTransient<FormCreateSupply>();
            services.AddTransient<FormSell>();
            services.AddTransient<FormReportPlaneComponents>();
            services.AddTransient<FormReportOrders>();
            services.AddTransient<FormReportGroupOrders>();
            services.AddTransient<FormReportShopPlanes>();
        }
    }
}